import { ExtrinsicEventsCallback, Hash } from '@moonbeam-network/xcm-utils';
export {
  ExtrinsicEvent,
  ExtrinsicEventsCallback,
  ExtrinsicFailedEvent,
  ExtrinsicSentEvent,
  ExtrinsicStatus,
  ExtrinsicSuccessEvent,
} from '@moonbeam-network/xcm-utils';
import { TransactionResponse } from '@ethersproject/abstract-provider';
import {
  AssetSymbol,
  Asset,
  WithdrawXTokensConfig,
  MoonbaseAssets,
  MoonbaseChains,
  MoonbeamAssets,
  MoonbeamChains,
  MoonriverAssets,
  MoonriverChains,
  ChainKey,
  AssetsMap,
  MoonChain,
  Chain,
  BalanceConfig,
  MinBalanceConfig,
  ExtrinsicConfig,
  XcmConfigBuilder,
} from '@moonbeam-network/xcm-config';
import { Signer } from 'ethers';
import { ApiPromise } from '@polkadot/api';
import {
  UnsubscribePromise,
  Signer as Signer$1,
  SubmittableExtrinsic,
} from '@polkadot/api/types';
import { AccountInfo } from '@polkadot/types/interfaces';
import {
  PalletAssetsAssetMetadata,
  PalletAssetsAssetAccount,
} from '@polkadot/types/lookup';
import { IKeyringPair } from '@polkadot/types/types';

declare class XTokensContract<Symbols extends AssetSymbol = AssetSymbol> {
  #private;
  readonly address: string;
  constructor(signer: Signer);
  transfer(
    account: string,
    amount: bigint,
    asset: Asset<Symbols>,
    config: WithdrawXTokensConfig<Symbols>,
    minAmount: bigint,
  ): Promise<TransactionResponse>;
  getTransferFees(
    account: string,
    amount: bigint,
    asset: Asset<Symbols>,
    config: WithdrawXTokensConfig<Symbols>,
  ): Promise<bigint>;
  private getGasPrice;
}

interface XcmSdkByChain {
  moonbase: MoonbaseXcmSdk;
  moonbeam: MoonbeamXcmSdk;
  moonriver: MoonriverXcmSdk;
}
type MoonXcmSdk = MoonbaseXcmSdk | MoonbeamXcmSdk | MoonriverXcmSdk;
type MoonbaseXcmSdk = XcmSdk<MoonbaseAssets, MoonbaseChains>;
type MoonbeamXcmSdk = XcmSdk<MoonbeamAssets, MoonbeamChains>;
type MoonriverXcmSdk = XcmSdk<MoonriverAssets, MoonriverChains>;
interface XcmSdk<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  symbols: Symbols[];
  assets: AssetsMap<Symbols>;
  moonAsset: Asset<Symbols>;
  moonChain: MoonChain;
  subscribeToAssetsBalanceInfo: (
    account: string,
    cb: (data: AssetBalanceInfo<Symbols>[]) => void,
  ) => UnsubscribePromise;
  deposit: (
    symbolOrAsset: Symbols | Asset<Symbols>,
  ) => XcmSdkDeposit<Symbols, ChainKeys>;
  withdraw: (
    symbolOrAsset: Symbols | Asset<Symbols>,
  ) => XcmSdkWithdraw<Symbols, ChainKeys>;
}
interface XcmSdkDeposit<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  chains: Chain<ChainKeys>[];
  from: (
    keyOrChain: ChainKeys | Chain<ChainKeys>,
  ) => XcmSdkDepositFrom<Symbols>;
}
interface XcmSdkWithdraw<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  chains: Chain<ChainKeys>[];
  to: (keyOrChain: ChainKeys | Chain<ChainKeys>) => XcmSdkWithdrawTo<Symbols>;
}
interface XcmSdkDepositFrom<Symbols extends AssetSymbol = AssetSymbol> {
  get: (
    account: string,
    sourceAccount: string | IKeyringPair,
    params?: DepositGetParams,
  ) => Promise<DepositTransferData<Symbols>>;
}
interface DepositGetParams {
  polkadotSigner?: Signer$1;
  primaryAccount?: string;
}
interface XcmSdkWithdrawTo<Symbols extends AssetSymbol = AssetSymbol> {
  get: (
    destinationAccount: string,
    params?: WithdrawGetParams,
  ) => Promise<WithdrawTransferData<Symbols>>;
}
interface WithdrawGetParams {
  ethersSigner?: Signer;
}
interface DepositTransferData<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  asset: AssetWithDecimals<Symbols>;
  existentialDeposit: bigint;
  min: bigint;
  moonChainFee: XcmFeeWithDecimals<Symbols>;
  native: AssetWithDecimals<Symbols>;
  origin: MoonChain | Chain<ChainKeys>;
  source: Chain<ChainKeys>;
  sourceBalance: bigint;
  sourceAssetDecimals: number;
  sourceFeeBalance?: Balance<Symbols>;
  sourceMinBalance: bigint;
  getFee: (amount?: bigint) => Promise<bigint>;
  send: (amount: bigint, cb?: ExtrinsicEventsCallback) => Promise<Hash>;
}
interface WithdrawTransferData<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  asset: AssetWithDecimals<Symbols>;
  destination: Chain<ChainKeys>;
  destinationBalance: bigint;
  destinationAssetDecimals: number;
  destinationFee: bigint;
  existentialDeposit: bigint;
  min: bigint;
  minXcmFeeAsset: XcmFeeWithDecimals<Symbols>;
  native: AssetWithDecimals<Symbols>;
  origin: MoonChain | Chain<ChainKeys>;
  originXcmFeeAssetBalance?: bigint;
  getFee: (amount: bigint) => Promise<bigint>;
  send: (amount: bigint, cb?: ExtrinsicEventsCallback) => Promise<Hash>;
}
interface AssetWithDecimals<Symbols extends AssetSymbol = AssetSymbol>
  extends Asset<Symbols> {
  decimals: number;
}
interface XcmFeeWithDecimals<Symbols extends AssetSymbol = AssetSymbol> {
  amount: bigint;
  decimals: number;
  symbol: Symbols;
}
interface Balance<Symbols extends string | AssetSymbol = AssetSymbol> {
  balance: bigint;
  decimals: number;
  symbol: Symbols;
}
interface SdkOptions {
  ethersSigner?: Signer;
  polkadotSigner?: Signer$1;
}

declare function init(options?: SdkOptions): XcmSdkByChain;

declare function isXcmSdkDeposit(
  config: XcmSdkDeposit | XcmSdkWithdraw,
): config is XcmSdkDeposit;
declare function isXcmSdkWithdraw(
  config: XcmSdkDeposit | XcmSdkWithdraw,
): config is XcmSdkWithdraw;
declare function isWithdrawTransferData(
  data?: WithdrawTransferData | DepositTransferData,
): data is WithdrawTransferData;
declare function isDepositTransferData(
  data?: WithdrawTransferData | DepositTransferData,
): data is DepositTransferData;

interface AssetBalanceInfo<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  asset: Asset<Symbols>;
  balance: Balance<string>;
  origin: Chain<ChainKeys> | MoonChain;
}
interface XCMType {
  Xcm: {
    parents: number;
    interior: unknown;
  };
}

declare class PolkadotService<
  Symbols extends AssetSymbol = AssetSymbol,
  ChainKeys extends ChainKey = ChainKey,
> {
  #private;
  constructor(api: ApiPromise);
  static create<
    Symbols extends AssetSymbol = AssetSymbol,
    ChainKeys extends ChainKey = ChainKey,
  >(ws: string): Promise<PolkadotService<Symbols, ChainKeys>>;
  getMetadata(): {
    decimals: number;
    symbol: Symbols;
    ss58Format: number;
    genesisHash: `0x${string}`;
  };
  getExistentialDeposit(): bigint;
  getAssetDecimals(asset: Asset<Symbols>): Promise<number>;
  getAssetMeta(asset: Asset<Symbols>): Promise<PalletAssetsAssetMetadata>;
  subscribeToAccountInfo(
    account: string,
    callback: (info: AccountInfo) => void,
  ): UnsubscribePromise;
  subscribeToBalance(
    account: string,
    callback: (balance: bigint) => void,
  ): UnsubscribePromise;
  getGenericBalance(
    account: string,
    { pallet, function: fn, getParams, path, calc }: BalanceConfig<Symbols>,
  ): Promise<bigint>;
  getAssetMinBalance({
    pallet,
    function: fn,
    params,
    path,
  }: MinBalanceConfig): Promise<bigint>;
  getXcmExtrinsic(
    account: string,
    amount: bigint,
    { pallet, extrinsic, getParams }: ExtrinsicConfig<Symbols>,
    fee?: bigint,
    primaryAccount?: string,
  ): SubmittableExtrinsic<'promise'>;
  getAssetFee(
    asset: Asset,
    weight: number,
    moonChain: MoonChain,
  ): Promise<bigint>;
  getUnitsPerSecond(asset: Asset, moonChain: MoonChain): Promise<bigint>;
  /**
   * this is only for Moon* chains
   */
  getAssetType(id: string): Promise<XCMType | undefined>;
  subscribeToAssetsBalanceInfo(
    account: string,
    config: XcmConfigBuilder<Symbols, ChainKeys>,
    callback: (data: AssetBalanceInfo<Symbols>[]) => void,
  ): UnsubscribePromise;
  getAssetsMetadata(
    assetsArray: Asset<Symbols>[],
  ): Promise<PalletAssetsAssetMetadata[]>;
  subscribeToAccountBalances(
    account: string,
    assetsArray: Asset<Symbols>[],
    callback: (balances: PalletAssetsAssetAccount[]) => void,
  ): UnsubscribePromise;
}

declare function createPolkadotServices<
  Symbols extends AssetSymbol,
  ChainKeys extends ChainKey,
>(ws: string[]): Promise<PolkadotService<Symbols, ChainKeys>[]>;

export {
  AssetBalanceInfo,
  AssetWithDecimals,
  Balance,
  DepositGetParams,
  DepositTransferData,
  MoonXcmSdk,
  MoonbaseXcmSdk,
  MoonbeamXcmSdk,
  MoonriverXcmSdk,
  PolkadotService,
  SdkOptions,
  WithdrawGetParams,
  WithdrawTransferData,
  XCMType,
  XTokensContract,
  XcmFeeWithDecimals,
  XcmSdk,
  XcmSdkByChain,
  XcmSdkDeposit,
  XcmSdkDepositFrom,
  XcmSdkWithdraw,
  XcmSdkWithdrawTo,
  createPolkadotServices,
  init,
  isDepositTransferData,
  isWithdrawTransferData,
  isXcmSdkDeposit,
  isXcmSdkWithdraw,
};
